Package: dsem
Type: Package
Title: Fit Dynamic Structural Equation Models
Version: 1.4.0
Date: 2025-01-10
Authors@R: 
    c(person(given = "James",
             family = "Thorson",
             role = c("aut", "cre"),
             email = "James.Thorson@noaa.gov")
    )
Imports: TMB, Matrix, sem, igraph, RTMB (>= 1.7.0), ggraph, ggplot2,
        grid, methods
Depends: R (>= 4.0.0),
Suggests: knitr, AER, phylopath, rmarkdown, reshape, gridExtra, dynlm,
        MARSS, ggpubr, vars, testthat, DHARMa
Enhances: rstan, tmbstan
LinkingTo: TMB, RcppEigen
Description: Applies dynamic structural equation models to time-series data
	with generic and simplified specification for simultaneous and lagged
	effects. Methods are described in Thorson et al. (2024)
	"Dynamic structural equation models synthesize ecosystem dynamics 
	constrained by ecological mechanisms."  
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.2
VignetteBuilder: knitr
LazyData: true
URL: https://james-thorson-noaa.github.io/dsem/
BugReports: https://github.com/James-Thorson-NOAA/dsem/issues
NeedsCompilation: yes
Packaged: 2025-01-13 21:57:11 UTC; James.Thorson
Author: James Thorson [aut, cre]
Maintainer: James Thorson <James.Thorson@noaa.gov>
Repository: CRAN
Date/Publication: 2025-01-13 22:30:04 UTC
Built: R 4.3.3; aarch64-apple-darwin20; 2025-01-28 21:50:46 UTC; unix
Archs: dsem.so.dSYM
