Package: dsem
Type: Package
Title: Dynamic Structural Equation Models
Version: 1.7.0
Date: 2025-09-15
Authors@R: 
    c(person(given = "James",
             family = "Thorson",
             role = c("aut", "cre"),
             email = "James.Thorson@noaa.gov",
             comment = c(ORCID = "0000-0001-7415-1010")),
      person(given = "Maurice",
             family = "Goodman",
             role = c("ctb"),
             email = "goodman.maurice@gmail.com",
             comment = c(ORCID = "0000-0002-6874-2313")),    
      person(given = "Wouter", 
 	     family = "van der Bijl",
             role = c("ctb"),
             comment = c(ORCID = "0000-0002-7366-1868", "template for d-separation test")),
      person(given = "Giovanni M.", 
	     family = "Marchetti", 
	     email = "giovanni.marchetti@unifi.it", 
             role = c("ctr"),
	     comment = c("creator of ggm, from which functions are copied to avoid dependency on package graph not on CRAN"))
    )
Imports: TMB, Matrix, sem, igraph, utils, RTMB (>= 1.7.0), ggraph,
        ggplot2, grid, methods, stats
Depends: R (>= 4.0.0),
Suggests: knitr, AER, phylopath, rmarkdown, reshape, gridExtra, dynlm,
        MARSS, ggpubr, vars, testthat, DHARMa
Enhances: rstan, tmbstan
LinkingTo: TMB, RcppEigen
Description: Applies dynamic structural equation models to time-series data
	with generic and simplified specification for simultaneous and lagged
	effects. Methods are described in Thorson et al. (2024)
	"Dynamic structural equation models synthesize ecosystem dynamics 
	constrained by ecological mechanisms."  
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
VignetteBuilder: knitr
LazyData: true
URL: https://james-thorson-noaa.github.io/dsem/
BugReports: https://github.com/James-Thorson-NOAA/dsem/issues
NeedsCompilation: yes
Packaged: 2025-09-16 00:03:53 UTC; James.Thorson
Author: James Thorson [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7415-1010>),
  Maurice Goodman [ctb] (ORCID: <https://orcid.org/0000-0002-6874-2313>),
  Wouter van der Bijl [ctb] (ORCID:
    <https://orcid.org/0000-0002-7366-1868>, template for d-separation
    test),
  Giovanni M. Marchetti [ctr] (creator of ggm, from which functions are
    copied to avoid dependency on package graph not on CRAN)
Maintainer: James Thorson <James.Thorson@noaa.gov>
Repository: CRAN
Date/Publication: 2025-09-16 02:40:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-02-16 03:56:16 UTC; unix
Archs: dsem.so.dSYM
